"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.defaultOptions = exports.TRANSITION_RANDOM = exports.TRANSITION_PROGRESSIVE = exports.TRANSITION_CONSTANT = void 0;

var _measureText2 = _interopRequireDefault(require("./measureText"));

var _rejectUndefined = _interopRequireDefault(require("./rejectUndefined"));

var _getCharHorizontalPos = _interopRequireDefault(require("./getCharHorizontalPos"));

var _getCharVerticalPos = _interopRequireDefault(require("./getCharVerticalPos"));

var _defaultConstants = require("../defaultConstants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const TRANSITION_CONSTANT = 'constant';
exports.TRANSITION_CONSTANT = TRANSITION_CONSTANT;
const TRANSITION_PROGRESSIVE = 'progressive';
exports.TRANSITION_PROGRESSIVE = TRANSITION_PROGRESSIVE;
const TRANSITION_RANDOM = 'random';
exports.TRANSITION_RANDOM = TRANSITION_RANDOM;
const defaultOptions = {
  width: _defaultConstants.DEFAULT_WIDTH,
  height: _defaultConstants.DEFAULT_HEIGHT,
  fontSize: _defaultConstants.DEFAULT_FONT_SIZE,
  lineHeight: _defaultConstants.DEFAULT_LINE_HEIGHT,
  fontFamily: _defaultConstants.DEFAULT_FONT_FAMILY,
  transitionStyle: _defaultConstants.DEFAULT_TRANSITION_STYLE,
  transitionSpeedMultiplier: _defaultConstants.DEFAULT_TRANSITION_SPEED_MULTIPLIER,
  transitionTimingFunction: _defaultConstants.DEFAULT_TRANSITION_TIMING_FUNCTION,
  charCenters: undefined,
  untidyOnHover: _defaultConstants.DEFAULT_UNTIDY_ON_HOVER,
  vertical: _defaultConstants.DEFAULT_VERTICAL
};
exports.defaultOptions = defaultOptions;

const generateTransition = (type, speedMultiplier, transitionTimingFunction, index = 0) => {
  switch (type) {
    case TRANSITION_CONSTANT:
      return `all ${1 * speedMultiplier}s ${transitionTimingFunction}`;

    case TRANSITION_PROGRESSIVE:
      return `all ${(1 + 0.15 * index) * speedMultiplier}s ${transitionTimingFunction}`;

    case TRANSITION_RANDOM:
      return `all ${(1 + Math.random()) * speedMultiplier}s ${transitionTimingFunction}`;

    default:
      return `all ${1 * speedMultiplier}s ${transitionTimingFunction} `;
  }
};
/**
 * Generates the styles for the wrapper and each of the text characters for the animation to happen.
 *
 * @param {string} text text to be rendered. It is used to generate a style for each character.
 * @param {object} [options={}] object with options
 * @param {string} [options.fontSize='20px'] fontSize of the text when tidy
 * @param {number} [options.lineHeight=1.3] lineHeight of the text when tidy.
 * @param {string} [options.fontFamily='Georgia'] fontFamily of the text.
 * @param {string} [options.transitionStyle='constant'] style of the transition animation. Values: 'constant' | 'progressive' | 'random'.
 * @param {string} [options.transitionSpeedMultiplier=1] speed multiplier for the transition. Default transitions take 1s. The multiplier can increase and decrease that.
 * @param {string} [transitionTimingFunction='ease'] is the transition timing function used on CSS transition. e.g. ease, ease-in, cubic-bezier(1, 0.24, 0.25, 1), ...
 * @param {boolean} [options.untidyOnHover=false] when true the behaviour is the opposite.
 * @param {boolean} [options.vertical=false] when true, the tidied text renders in vertical.
 * @param {Array<{ x: number, y: number }>} [options.charCenters=undefined] position of the characters before the user hovers them.
 * @returns {{[styleName]: styles}} returns a object with the JSS styles for the wrapper and each character. This object needs to be consumed
 * by the `withStyles` HoC from Material-UI to generate the CSS. The `injectStyles` HoC from JSS can also be used.
 */


const createStyles = (text, options = defaultOptions) => () => {
  const _defaultOptions$rejec = _objectSpread({}, defaultOptions, (0, _rejectUndefined.default)(options)),
        width = _defaultOptions$rejec.width,
        height = _defaultOptions$rejec.height,
        fontSize = _defaultOptions$rejec.fontSize,
        lineHeight = _defaultOptions$rejec.lineHeight,
        fontFamily = _defaultOptions$rejec.fontFamily,
        transitionStyle = _defaultOptions$rejec.transitionStyle,
        transitionSpeedMultiplier = _defaultOptions$rejec.transitionSpeedMultiplier,
        transitionTimingFunction = _defaultOptions$rejec.transitionTimingFunction,
        charCenters = _defaultOptions$rejec.charCenters,
        untidyOnHover = _defaultOptions$rejec.untidyOnHover,
        vertical = _defaultOptions$rejec.vertical;

  const textAsArray = text.split('');

  const _measureText = (0, _measureText2.default)(text, {
    fontFamily,
    fontSize,
    lineHeight
  }),
        textWidth = _measureText.width.value;

  const textCharWidths = textAsArray.map(char => (0, _measureText2.default)(char, {
    fontFamily,
    fontSize,
    lineHeight
  }).width.value);
  const textCharHeights = textAsArray.map(char => (0, _measureText2.default)(char, {
    fontFamily,
    fontSize,
    lineHeight
  }).height.value); // Calculate total text Height when rendered vertically

  const textHeight = textCharHeights.reduce((acc, value) => acc + value, 0);

  const charsHorizontalStyle = index => _objectSpread({
    position: 'absolute',
    transition: generateTransition(transitionStyle, transitionSpeedMultiplier, transitionTimingFunction, index),
    top: '50%',
    transform: `rotate(0deg)`,
    fontSize
  }, (0, _getCharHorizontalPos.default)(index, textWidth, textCharWidths));

  const charsVerticalStyle = index => _objectSpread({
    position: 'absolute',
    transition: generateTransition(transitionStyle, transitionSpeedMultiplier, transitionTimingFunction, index),
    transform: `rotate(0deg)`,
    fontSize
  }, (0, _getCharVerticalPos.default)(index, textHeight, textCharHeights, textCharWidths));

  const charsUntidyStyle = index => ({
    position: 'absolute',
    transition: generateTransition(transitionStyle, transitionSpeedMultiplier, transitionTimingFunction, index),
    left: `${charCenters[index].x / width * 100}%`,
    top: `${charCenters[index].y / height * 100}%`,
    transform: `rotate(${Math.random() * 90 - Math.random() * 90}deg)`
  });

  const charsHoverStyle = textAsArray.reduce((acc, char, index) => {
    if (untidyOnHover && charCenters) {
      acc[`&:hover $char-${index}`] = _objectSpread({}, charsUntidyStyle(index), {
        fontSize: 'inherit'
      });
      return acc;
    }

    acc[`&:hover $char-${index}`] = vertical ? _objectSpread({}, charsVerticalStyle(index)) : _objectSpread({}, charsHorizontalStyle(index));
    return acc;
  }, {});
  const charsDefaultStyle = textAsArray.reduce((acc, char, index) => {
    if (!untidyOnHover && charCenters) {
      acc[`char-${index}`] = _objectSpread({}, charsUntidyStyle(index));
      return acc;
    }

    acc[`char-${index}`] = vertical ? _objectSpread({}, charsVerticalStyle(index)) : _objectSpread({}, charsHorizontalStyle(index));
    return acc;
  }, {});
  return _objectSpread({
    wrapper: _objectSpread({
      width: '100%',
      height: '100%',
      position: 'relative',
      fontSize: '50px'
    }, charsHoverStyle)
  }, charsDefaultStyle);
};

var _default = createStyles;
exports.default = _default;